
  /**
   * @license
   * author: dudiq
   * jr-translate.js v2.0.0
   * Released under the MIT license.
   */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const n = typeof navigator === 'object' ? navigator : undefined;
const DEF_LANG = 'en';
class Languages {
    defaultLanguage = DEF_LANG;
    currentLanguage = ((n && n.language) || '').substring(0, 2).toLowerCase() || DEF_LANG;
    languageList = [DEF_LANG];
    setDefaultLanguage(lang) {
        this.defaultLanguage = lang;
    }
    setLanguage(value) {
        this.currentLanguage = value;
    }
    setLanguageList(value) {
        this.languageList = value;
    }
}
const langs = new Languages();

const isClient = typeof window !== 'undefined';
const canUseFormatter = isClient ? !!window.Intl : true; // support for old browsers;
const map = {};
function getNumByLang(lang, number) {
    if (map[lang] !== false) {
        // init
        map[lang] = canUseFormatter ? new Intl.NumberFormat(lang) : false;
    }
    const node = map[lang];
    if (isNaN(number) || node === false) {
        return number + '';
    }
    return node.format(number);
}
function numberFormat(lang, number, precision) {
    if (precision === undefined) {
        return getNumByLang(lang, number);
    }
    const dx = Math.pow(10, precision);
    const bigNum = Math.floor(number * dx);
    const floatDx = 1 / (dx * 10);
    const int = Math.floor(number);
    let float = Math.floor(bigNum - int * dx) / dx;
    float += floatDx;
    const localeInt = getNumByLang(lang, int);
    let localeFloat = getNumByLang(lang, float);
    localeFloat = localeFloat.substring(1, localeFloat.length - 1);
    return localeInt + (localeFloat.length > 1 ? localeFloat : '');
}
function getNumber(lang, number, precision) {
    if (typeof lang === 'number') {
        return numberFormat(langs.currentLanguage, lang, number);
    }
    return numberFormat(lang, number, precision);
}

const pluEn = {
    lang: 'en',
    getIndex: (num) => {
        return num !== 1 ? 1 : 0;
    },
};

const pluRu = {
    lang: 'ru',
    getIndex: (num) => {
        return num % 10 == 1 && num % 100 != 11
            ? 0
            : num % 10 >= 2 && num % 10 <= 4 && (num % 100 < 10 || num % 100 >= 20)
                ? 1
                : 2;
    },
};

const pluEs = {
    lang: 'es',
    getIndex: (num) => {
        return num !== 1 ? 1 : 0;
    },
};

const pluFr = {
    lang: 'fr',
    getIndex: (num) => {
        return num > 1 ? 1 : 0;
    },
};

const canUseConsole = typeof console == 'object';
let isEnabled = true;
function toggleLogError(value) {
    isEnabled = value;
}
function logError(...args) {
    if (!isEnabled)
        return;
    if (!canUseConsole)
        return;
    console.error('#translate:', ...args);
}

const pluralMap = {};
function addPlural$1(plu) {
    if (pluralMap[plu.lang]) {
        logError(`trying register already defined plural - ${plu.lang}`);
        return;
    }
    pluralMap[plu.lang] = plu;
}
function plurals(lang, pluralFields, data) {
    const plu = pluralMap[lang];
    let ret = pluralFields.key || '';
    const isObject = typeof data === 'object';
    for (const key in pluralFields) {
        if (key === 'key')
            continue;
        const num = (isObject ? data[key] : data);
        const pluArray = pluralFields[key];
        const index = plu.getIndex(num);
        let pluStr = pluArray ? pluArray[index] : '';
        const numFormatted = numberFormat(lang, num);
        pluStr = pluStr.replace('{#}', numFormatted);
        ret = ret.replace(`{#${key}}`, pluStr);
    }
    if (!isObject)
        return ret;
    for (const key in data) {
        if (Object.prototype.hasOwnProperty.call(pluralFields, key))
            continue;
        const val = data[key];
        ret = ret.replace(`{#${key}}`, `${val}`);
    }
    return ret;
}
plurals.addPlural = function (plu) {
    if (!Array.isArray(plu)) {
        addPlural$1(plu);
        return;
    }
    plu.forEach(function (item) {
        addPlural$1(item);
    });
};
// default plurals
plurals.addPlural([pluEn, pluRu, pluEs, pluFr]);

const hashMap = {};
const DEF_SEPARATOR = '.';
function applyData(str, data) {
    if (!data)
        return str;
    if (typeof data !== 'object') {
        return str.replace('{#}', `${data}`);
    }
    let ret = str;
    for (const key in data) {
        const val = data[key];
        ret = ret.replace(`{#${key}}`, `${val}`);
    }
    return ret;
}
function onBlockParsed(langsList, hash, block) {
    if (Array.isArray(block)) {
        setToMap(block, langsList, hash, block);
    }
    else {
        setToMap(langsList, langsList, hash, block);
    }
}
function processKeyTree(nestedTree, langsList, hash, separator) {
    if (typeof nestedTree !== 'object' || Array.isArray(nestedTree)) {
        onBlockParsed(langsList, hash, nestedTree);
        return;
    }
    for (const key in nestedTree) {
        const newHash = hash ? hash + separator + key : key;
        const block = nestedTree[key];
        processKeyTree(block, langsList, newHash, separator);
    }
}
function initLangs(lang) {
    hashMap[lang] = hashMap[lang] || {};
}
function parseBlock(block, langsList) {
    langsList.forEach(initLangs);
    processKeyTree(block, langsList, '', DEF_SEPARATOR);
}
function setToMap(list, langsList, hash, data) {
    for (let i = 0, l = list.length; i < l; i++) {
        const lang = langsList[i] || langs.defaultLanguage;
        const item = Array.isArray(data) ? data[i] : data;
        const langMap = hashMap[lang];
        if (langMap[hash]) {
            logError('trying to register already defined HASH lang!', hash);
        }
        else {
            hashMap[lang][hash] = item;
        }
    }
}
function getData(lang, field, data) {
    if (typeof field === 'object') {
        return plurals(lang, field, data);
    }
    return applyData(field, data);
}
function getBlockByKey(lang, key, data) {
    const translateBlock = hashMap[lang];
    if (translateBlock && translateBlock[key] !== undefined) {
        return getData(lang, translateBlock[key], data);
    }
    return;
}
function translate(key, data) {
    return translateByLang(langs.currentLanguage, key, data);
}
function translateByLang(lang, key, data) {
    const result = getBlockByKey(lang, key, data);
    if (result !== undefined) {
        return result;
    }
    logError(`not defined block [${key}] in ${lang} lang`);
    if (lang === langs.defaultLanguage) {
        return key;
    }
    const defaultResult = getBlockByKey(langs.defaultLanguage, key, data);
    if (defaultResult === undefined) {
        return key;
    }
    return defaultResult;
}
function addBlock(block) {
    parseBlock(block.data, block.langs || langs.languageList);
}

function getLang() {
    return langs.currentLanguage;
}
function setLangs(list) {
    langs.setLanguageList(list);
}
function setLang(lang) {
    langs.setLanguage(lang);
}
function setDefaultLanguage(lang) {
    langs.setDefaultLanguage(lang);
}
function addPlural(plu) {
    plurals.addPlural(plu);
}

exports.addBlock = addBlock;
exports.addPlural = addPlural;
exports.getLang = getLang;
exports.getNumber = getNumber;
exports.setDefaultLanguage = setDefaultLanguage;
exports.setLang = setLang;
exports.setLangs = setLangs;
exports.toggleLogError = toggleLogError;
exports.translate = translate;
exports.translateByLang = translateByLang;
//# sourceMappingURL=translate.cjs.js.map
